;; ai-lab-i.scm - Solutions to AI-LAB-I exercises
;; Version 1.0
;;
;; Copyright (C) 2004 Frank C Langbein <frank@langbein.org>.
;; All rights reserved.
;;
;; This program is free software; you can redistribute it and/or modify it 
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this program; if not, write to the Free Software Foundation, Inc.,
;; 675 Mass Ave, Cambridge, MA 02139, USA.

;; Note that there are alternatives and these solutions are not optimised.


;; 1 Simple functions

; 1(a) as symbol

(define expression-1a (/ (+ 5 (/ 3 4)) (+ (* 7 10) 9)))

; 1(b)

(define function-1b 
  (lambda (x y)
    (/ (+ 5 (/ 3 y)) (+ 1 (* x x)))))

; 1(c)

(define factorial
  (lambda (n)
    (if (= n 0) 1
	(* n (factorial (- n 1))))))

; 1(d)

(define exp
  (lambda (x t) (help-exp x t 0)))

(define help-exp
  (lambda (x t n)
    (let ((d (/ (expt x n) (factorial n))))
      (if (< d t) d
	  (+ d (help-exp x t (+ n 1)))))))

; or with named let (recursion to implement loop):

(define exponential
  (lambda (x t) 
    (let loop ((n 0))
      (let ((d (/ (expt x n) (factorial n))))
	(if (< d t) d
	    (+ d (loop (+ n 1))))))))


;; 2 8-Puzzle

; 2(a)

; 8 Puzzle lables and values:
;
; 0 1 2    n0 n1 n2
; 3 4 5    n3 n4 n5   nX \in {0, ..., 8} where 0 indicates empty
; 6 7 8    n6 n7 n8
;
; Put them in a list in this order for human-readable input
; '(n0 n1 n2 n3 n4 n5 n6 n7 n8)
;
; Convert states to number as internal format:
;
; n0 * 9^0 + n1 * 9^1 + n2 * 9^2 +
; n3 * 9^3 + n4 * 9^4 + n5 * 9^5 +
; n6 * 9^6 + n7 * 9^7 + n8 * 9^8

(define to-internal
  (lambda (list)
    (let loop ((E 1) (l list))
      (if (null? l) 0
	  (+ (* (car l) E) 
	     (loop (* E 9) (cdr l)))))))

(define display-puzzle
  (lambda (state)
    (newline)
    (let loop ((rep state) (pos 0))
      (if (> pos 8) #t
	  (begin
	    (display (modulo rep 9))
	    (if (or (= pos 2) (= pos 5) (= pos 8)) (newline))
	    (loop (quotient rep 9) (+ pos 1)))))))

; To display goal state: 
;  (display-puzzle (to-internal '(1 2 3 8 0 4 7 6 5)))

; 2(b)

(define goal-state (to-internal '(1 2 3 8 0 4 7 6 5)))

(define is-goal?
  (lambda (state) (= state goal-state)))

; 2(c)

(define at-pos
  (lambda (state pos)
    (modulo (integer-floor state (expt 9 pos)) 9)))

(define empty-square
  (lambda (state)
    (let loop ((pos 0))
      (if (= (at-pos state pos) 0) pos
	  (loop (+ pos 1))))))

(define legal-moves
  (lambda (pos) (cond ((= pos 0) '(1 3))
		      ((= pos 1) '(0 2 4))
		      ((= pos 2) '(1 5))
		      ((= pos 3) '(0 4 6))
		      ((= pos 4) '(1 3 5 7))
		      ((= pos 5) '(2 4 8))
		      ((= pos 6) '(3 7))
		      ((= pos 7) '(4 6 8))
		      ((= pos 8) '(5 7)))))

; Move empty position: change number indicating state
; by different between the exponentials of 9 for the
; the different positions multiplied by the number on
; the tile moved.
(define apply-move 
  (lambda (state from to)
    (+ state (* (at-pos state to) (- (expt 9 from) (expt 9 to))))))

(define successors
  (lambda (state)
    (let ((pos-empty (empty-square state))
	  (res '()))
      (let loop ((moves (legal-moves pos-empty)))
	(if (null? moves) ()
	    (cons (list (car moves) (apply-move state pos-empty (car moves))
			state)
		  (loop (cdr moves))))))))

; 2(d)

(define cycle?
  (lambda (state list)
    (cond ((null? list) #f)
	  ((= (cadar list) (cadr state)) #t)
	  (else (cycle? state (cdr list))))))

(define cycle-append
  (lambda (new queue visited)
    (let loop ((l new))
      (cond ((null? l) queue)
	    ((or (cycle? (car l) queue)
		 (cycle? (car l) visited))
	     (loop (cdr l)))
	    (else
	     (append (loop (cdr l)) (list (car l))))))))

(define display-move
  (lambda (move)
    (display-puzzle (cadr move))
    (if (< (car move) 9) (begin (display "Move: ") (display (car move))))
    #t))

(define display-solution
  (lambda (visited state)
    (if (= (caddr state) 0) #t
	(let loop ((l visited))
	  (if (= (caddr state) (cadar l))
	      (begin (display-move (car l))
		     (display-solution visited (car l)))
	      (loop (cdr l)))))))

; Breadth first search (prints out solution in reverse!)  
; The lists contain the states as lists of the following form:
;    (MOVE STATE PREVIOUS) 
; where PREVIOUS is the state expanded by applying the operator MOVE
; to get the state STATE
(define breadth-first
  (lambda (state)
    (let ((visited ()))
      (let main-loop ((queue (list (list 9 state 0))))
	
	(let ((top (car queue))
	      (rest (cdr queue)))
	  
	  (set! visited (cons top visited))
	  
	  (if (is-goal? (cadr top))
	      (begin (display-move top)
		     (display-solution visited top))
	      (main-loop (cycle-append (successors (cadr top))
				       rest visited)))
	  
	  )))))

; Solve puzzle using breadth first:
;
;(breadth-first (to-internal '(8 1 3 7 2 4 0 6 5)))


; 2(e)

; Manhatten distance of tile from intended position

(define pos-location
  (lambda (pos)
    (cond ((= pos 0) (cons 0 0))
	  ((= pos 1) (cons 1 0))
	  ((= pos 2) (cons 2 0))
	  ((= pos 3) (cons 0 1))
	  ((= pos 4) (cons 1 1))
	  ((= pos 5) (cons 2 1))
	  ((= pos 6) (cons 0 2))
	  ((= pos 7) (cons 1 2))
	  ((= pos 8) (cons 2 2)))))

(define tile-location
  (lambda (pos)
    (cond ((= pos 1) (cons 0 0))
	  ((= pos 2) (cons 1 0))
	  ((= pos 3) (cons 2 0))
	  ((= pos 4) (cons 2 1))
	  ((= pos 5) (cons 2 2))
	  ((= pos 6) (cons 1 2))
	  ((= pos 7) (cons 0 2))
	  ((= pos 8) (cons 0 1)))))

(define m-dist 
  (lambda (tile pos)
    (let ((a (tile-location tile))
	  (b (pos-location pos)))
      (+ (abs (- (car a) (car b)))
	 (abs (- (cdr a) (cdr b)))))))

(define manhatten
  (lambda (state)
    (let loop ((pos 0))
      (if (> pos 8) 0
	  (let ((tile (at-pos state pos)))
	    ( + (if (= tile 0) 0 (m-dist tile pos))
		(loop (+ pos 1))))))))

(define eval
  (lambda (l cost)
    (if (null? l) ()
	(cons (append (car l) (list (+ (manhatten (cadar l)) cost)))
	      (eval (cdr l) cost)))))

(define insert-sorted 
  (lambda (l s)
    (let loop ((r l))
      (if (null? r) (list s)
	  (if (> (cadddr (car r)) (cadddr s))
	      (cons s r)
	      (cons (car r) (loop (cdr r))))))))

(define cycle-sort-append
  (lambda (new queue visited cost)
    (let loop ((l (eval new cost)))
      (cond ((null? l) queue)
	    ((or (cycle? (car l) queue)
		 (cycle? (car l) visited))
	     (loop (cdr l)))
	    (else
	     (insert-sorted (loop (cdr l)) (car l)))))))

; A^* search (prints out solution in reverse!)
; The lists contain the states as lists of the following form:
;    (MOVE STATE PREVIOUS COST)
; where PREVIOUS is the state expanded by applying the operator MOVE
; to get the state STATE and COST is the evaluation of the state STATE
; (sum of heuristic function + real costs to get to state)
(define a-star
  (lambda (state)
    (let ((visited ()))
      (let main-loop ((queue (list (list 9 state 0 0))))
	
	(let ((top (car queue))
	      (rest (cdr queue)))
	  
	  (set! visited (cons top visited))
	  
	  (if (is-goal? (cadr top))
	      (begin (display-move top)
		     (display-solution visited top))
	      (main-loop (cycle-sort-append (successors (cadr top))
					    rest visited
					    (cadddr top))))

	  )))))

; Solve puzzle using A^*:
;
;(a-star (to-internal '(8 3 2 6 0 1 5 7 4)))
;
; this may take a while, but it's a lot faster than breadth-first:
;
;(breadth-first (to-internal '(8 3 2 6 0 1 5 7 4)))
