// xmath.hxx: Some useful math extensions.
// 
// Copyright (C) 1998-2003 Frank Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// $Id: xmath.hxx,v 2.3 2003/09/01 12:30:26 scmfcl Exp $

#ifndef XMATH_HXX
#define XMATH_HXX 1

// SiL precision
//
// Precision used to determine zero values, equalities, etc. As far as
// possible this should also be used the the modules.
extern double EPSILON;
extern double EPSILON2;

// PI constant.
//
// Value of the PI constant. If this is not already defined by some
// standard header file, we define it here.
#ifndef PI
#define PI 3.14159265358979323846
#endif // PI

// Convert deg to rad.
//
// Inline function to convert an angle from deg to rad.
inline double torad (double x)
{
  return (PI / 180.0) * x;
}

inline unsigned long min (unsigned long x, unsigned long y)
{
  return (x < y) ? x : y;
}

inline unsigned long max (unsigned long x, unsigned long y)
{
  return (x > y) ? x : y;
}

inline unsigned long min (long x, long y)
{
  return (x < y) ? x : y;
}

inline unsigned long max (long x, long y)
{
  return (x > y) ? x : y;
}

inline int min (int x, int y)
{
  return (x < y) ? x : y;
}

inline int max (int x, int y)
{
  return (x > y) ? x : y;
}

inline double min (double x, double y)
{
  return (x < y) ? x : y;
}

inline double max (double x, double y)
{
  return (x > y) ? x : y;
}

// Normalize a 3d GLfloat vector.
//
// Normalize a three-dimensional vector of the type GLfloat. The
// original vector is overwritten by the result.
void normalize (GLfloat * v);

// Normalized cross product of two 3d GLfloat vectors.
//
// Compute the cross product of v1 and v2 and store the normalized
// result in cp.
void ncrossprod (const GLfloat * v1, const GLfloat * v2, GLfloat * cp);

#endif // !XMATH_HXX

/// Local Variables:        ***
/// compile-command: "make" ***
/// End:                    ***
