// src/main.cxx
// PF main.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#include "config-pf.h"

ID(PF, pf_main, "$Id: main.c, v1.0 2003/09/01 scmfcl Exp $");

#include <unistd.h>
#define GNU_SOURCE
#include <getopt.h>

#include <gtkmm.h>

#include <iostream>

#include "pf/ui.hxx"

/* Print version information, etc. */
void
print_version (char const *package_name,
	       char const *package_version,
	       char const *program_name,
	       char const *program_version)
{
  assert (package_name != NULL);
  assert (package_version != NULL);

  if (program_name) {
    printf ("%c%s", toupper (*program_name), program_name + 1);
    if (program_version) {
      printf (" %s", program_version);
    }
    printf (" - ");
  }

  printf ("%c%s %s", toupper (*package_name), package_name + 1,
	  package_version);
  if (index (package_version, '.') == NULL) {
    printf (" (development snapshot)");
  } else if (index (package_version, ':') != NULL) {
    printf (" (testing)");
  }
  
  printf ("\nCopyright (C) 2004 Frank C. Langbein <frank@langbein.org>\n\n");
  printf ("Website: http://www.langbein.org/teaching/ai/\n");
  printf ("This is free software; see the source for copying conditions.  There is NO\n");
  printf ("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
  
  return;
}

/* Display command line interface help. */
void
print_help (char const *executable_name)
{
  printf ("Usage: %s [OPTION]... PROJECT_TYPE\n", executable_name);
  printf ("A* path finding demonstration.\n\n");
  printf ("Miscellaneous:\n");
  printf ("      --help                 display this help and exit\n");
  printf ("      --version              output version information and exit\n\n");
  printf ("Report bugs to <%s>.\n", PACKAGE_BUGREPORT);

  return;
}

/* Main program. */
int
main (int argc,
      char **argv)
{
  Gtk::Main kit (argc, argv);

  int show_help = 0;
  int show_version = 0;
  int error = 0;

  struct option long_options[] = {
    { "help",         no_argument,       &show_help,    1   },
    { "version",      no_argument,       &show_version, 1   }
  };

  while (1) {
    int option_index = 0;

    int c = getopt_long (argc, argv, "clo:t", long_options, &option_index);
    
    if (c == -1)
      break;

    switch (c) {

    case 0:
      break;

    default:
      error = 1;
      
    }
  }

  if (show_help) {
    print_help (argv[0]);
    exit (EXIT_SUCCESS);
  }

  if (show_version) {
    print_version (PACKAGE_NAME, PACKAGE_VERSION, NULL, NULL);
    exit (EXIT_SUCCESS);
  }

  if (error != 0) {
    fprintf (stderr, "Try `%s --help' for more information.\n", argv[0]);
    exit (EXIT_FAILURE);
  }

  PF::UI window;
  Gtk::Main::run (window);

  return 0;
}
