// src/ui.cxx
// PF main.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#include "config-pf.h"

ID(PF, pf_ui, "$Id: main.c, v1.0 2003/09/01 scmfcl Exp $");

#include "pf/ui.hxx"
#include "pf/map.hxx"
#include "pf/astar.hxx"

PF::UI::UI () 
  : map_ (new Map (50, 50, OCEAN, 2, 4, 6, 5)) {
  set_border_width (0);
  resize (50 * 10, 50 * 10);
  map_area_ = new MapArea (map_);
  add (*map_area_);
  map_area_->show ();
  
  PF::AStar a (1UL, 1UL, 5UL, 5UL, map_);
  a.search ();
}

PF::UI::~UI () {
  delete map_area_;
  delete map_;
}

PF::UI::MapArea::MapArea (Map *map) 
  : map_ (map) {
  Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap ();
  ocean_ = Gdk::Color ("darkblue");
  mountain_ = Gdk::Color ("grey");
  forest_ = Gdk::Color ("darkgreen");
  plane_ = Gdk::Color ("green");
  path_ = Gdk::Color ("black");
  solution_ = Gdk::Color ("red");

  colormap->alloc_color (ocean_);
  colormap->alloc_color (mountain_);
  colormap->alloc_color (forest_);
  colormap->alloc_color (plane_);
  colormap->alloc_color (path_);
  colormap->alloc_color (solution_);
}

PF::UI::MapArea::~MapArea () {
}

void PF::UI::MapArea::on_realize () {
  Gtk::DrawingArea::on_realize ();
  Glib::RefPtr<Gdk::Window> window (get_window ());
  gc_ = Gdk::GC::create (window);
  window->set_background (ocean_);
  window->clear ();
}

bool PF::UI::MapArea::on_expose_event (GdkEventExpose *e) {
  Glib::RefPtr<Gdk::Window> window (get_window ());
  window->clear ();
  for (unsigned long l = 0; l < map_->size_y (); ++l) {
    for (unsigned long k = 0; k < map_->size_x (); ++k) {
      switch ((*map_) (k, l)) {
      case OCEAN:
	gc_->set_foreground (ocean_);
	break;
      case MOUNTAIN:
	gc_->set_foreground (mountain_);
	break;
      case FOREST:
	gc_->set_foreground (forest_);
	break;
      default:
	gc_->set_foreground (plane_);
	break;
      }
      window->draw_rectangle (gc_, 1, k * 10, l * 10, 10, 10);
    }
  }

  gc_->set_foreground (path_);
  unsigned long px (0);
  unsigned long py (0);
  for (unsigned long l = 0; l < map_->size_y (); ++l) {
    for (unsigned long k = 0; k < map_->size_x (); ++k) {
      map_->prev (k, l, px, py);
      if (px < map_->size_x ()) {
	window->draw_line (gc_, k * 10 + 5, l * 10 + 5,
			   px * 10 + 5, py * 10 + 5);
      }
    }
  }

  return true;
}
